package java.awt.image;

import java.util.Hashtable;

public class FilteredImageSource implements ImageProducer
{
    ImageProducer src;
    ImageFilter filter;
    private Hashtable proxies;
    
    public FilteredImageSource(ImageProducer src, ImageFilter filter) {
        this.src = src;
        this.filter = filter;
    }
    
    @Override
    public synchronized void addConsumer(ImageConsumer imageConsumer) {
        if (this.proxies == null) {
            this.proxies = new Hashtable();
        }
        if (!this.proxies.containsKey(imageConsumer)) {
            ImageFilter filterInstance = this.filter.getFilterInstance(imageConsumer);
            this.proxies.put(imageConsumer, filterInstance);
            this.src.addConsumer(filterInstance);
        }
    }
    
    @Override
    public synchronized boolean isConsumer(ImageConsumer key) {
        return this.proxies != null && this.proxies.containsKey(key);
    }
    
    @Override
    public synchronized void removeConsumer(ImageConsumer imageConsumer) {
        if (this.proxies != null) {
            ImageFilter imageFilter = (ImageFilter)this.proxies.get(imageConsumer);
            if (imageFilter != null) {
                this.src.removeConsumer(imageFilter);
                this.proxies.remove(imageConsumer);
                if (this.proxies.isEmpty()) {
                    this.proxies = null;
                }
            }
        }
    }
    
    @Override
    public synchronized void startProduction(ImageConsumer imageConsumer) {
        if (this.proxies == null) {
            this.proxies = new Hashtable();
        }
        ImageFilter filterInstance = (ImageFilter)this.proxies.get(imageConsumer);
        if (filterInstance == null) {
            filterInstance = this.filter.getFilterInstance(imageConsumer);
            this.proxies.put(imageConsumer, filterInstance);
        }
        this.src.startProduction(filterInstance);
    }
    
    @Override
    public synchronized void requestTopDownLeftRightResend(ImageConsumer key) {
        if (this.proxies != null) {
            ImageFilter imageFilter = (ImageFilter)this.proxies.get(key);
            if (imageFilter != null) {
                imageFilter.resendTopDownLeftRight(this.src);
            }
        }
    }
}
